"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Discussions = void 0;
const pineapple_1 = require("@snapshot-labs/pineapple");
const constants_1 = require("../../constants");
const agent_1 = require("../agent");
const types_1 = require("./types");
class Discussions extends agent_1.Agent {
    async addCategory({ name, about, parent }) {
        const { cid } = await (0, pineapple_1.pin)({ name, about });
        const metadataURI = `ipfs://${cid}`;
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.addCategoryTypes, {
            parent,
            metadata_uri: metadataURI
        });
        const args = [parent, metadataURI, sig];
        return await this.client.invoke('discussions', 'add_category', args);
    }
    async editCategory({ category, name, about }) {
        const { cid } = await (0, pineapple_1.pin)({ name, about });
        const metadataURI = `ipfs://${cid}`;
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.editCategoryTypes, {
            category,
            metadata_uri: metadataURI
        });
        const args = [category, metadataURI, sig];
        return await this.client.invoke('discussions', 'edit_category', args);
    }
    async removeCategory({ category }) {
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.removeCategoryTypes, { category });
        const args = [category, sig];
        return await this.client.invoke('discussions', 'remove_category', args);
    }
    async addTopic({ title, content, author, category, parent }) {
        const { cid } = await (0, pineapple_1.pin)({ title, content });
        const metadataURI = `ipfs://${cid}`;
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.addTopicTypes, {
            author,
            category,
            parent,
            metadata_uri: metadataURI
        });
        const args = [author, category, parent, metadataURI, sig];
        return await this.client.invoke('discussions', 'add_topic', args);
    }
    async editTopic({ topic, title, content }) {
        const { cid } = await (0, pineapple_1.pin)({ title, content });
        const metadataURI = `ipfs://${cid}`;
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.editTopicTypes, {
            topic,
            metadata_uri: metadataURI
        });
        const args = [topic, metadataURI, sig];
        return await this.client.invoke('discussions', 'edit_topic', args);
    }
    async removeTopic({ topic }) {
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.removeTopicTypes, { topic });
        const args = [topic, sig];
        return await this.client.invoke('discussions', 'remove_topic', args);
    }
    async pinTopic({ topic }) {
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.pinTopicTypes, { topic });
        const args = [topic, sig];
        return await this.client.invoke('discussions', 'pin_topic', args);
    }
    async unpinTopic({ topic }) {
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.unpinTopicTypes, { topic });
        const args = [topic, sig];
        return await this.client.invoke('discussions', 'unpin_topic', args);
    }
    async vote({ voter, topic, choice }) {
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.voteTypes, { voter, topic, choice });
        const args = [voter, topic, choice, sig];
        return await this.client.invoke('discussions', 'vote', args);
    }
    async unvote({ voter, topic }) {
        const sig = await this.client.sign(constants_1.DOMAIN, types_1.unvoteTypes, { voter, topic });
        const args = [voter, topic, sig];
        return await this.client.invoke('discussions', 'unvote', args);
    }
}
exports.Discussions = Discussions;
