"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Client = void 0;
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const profiles_1 = require("./agents/profiles");
const discussions_1 = require("./agents/discussions");
const constants_1 = require("./constants");
class Client {
    signer;
    url;
    profiles;
    discussions;
    constructor(options) {
        this.url = options?.url || constants_1.HIGHLIGHT_TESTNET;
        this.signer = options?.signer;
        this.discussions = new discussions_1.Discussions(this);
        this.profiles = new profiles_1.Profiles(this);
    }
    setSigner(signer) {
        this.signer = signer;
    }
    async sign(domain, types, message) {
        if (!this.signer)
            throw new Error('signer is required');
        return await this.signer._signTypedData(domain, types, message);
    }
    async invoke(agent, method, args) {
        const init = {
            method: 'POST',
            headers: {
                Accept: 'application/json',
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                jsonrpc: '2.0',
                method: 'relay',
                params: {
                    messages: [
                        {
                            type: 'INVOKE_FUNCTION',
                            payload: {
                                agent,
                                method,
                                args
                            }
                        }
                    ]
                },
                id: null
            })
        };
        try {
            const res = await (0, cross_fetch_1.default)(this.url, init);
            return await res.json();
        }
        catch (e) {
            console.log('invoke failed', e);
        }
    }
}
exports.Client = Client;
